from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.internal.null_value import NullValue

from typing import Union

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.folder_full import FolderFull

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.items import Items

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData

from box_sdk_gen.internal.utils import DateTime


class GetFolderByIdSort(str, Enum):
    ID = 'id'
    NAME = 'name'
    DATE = 'date'
    SIZE = 'size'


class GetFolderByIdDirection(str, Enum):
    ASC = 'ASC'
    DESC = 'DESC'


class UpdateFolderByIdSyncState(str, Enum):
    SYNCED = 'synced'
    NOT_SYNCED = 'not_synced'
    PARTIALLY_SYNCED = 'partially_synced'


class UpdateFolderByIdParent(BaseObject):
    def __init__(
        self, *, id: Optional[str] = None, user_id: Optional[str] = None, **kwargs
    ):
        """
        :param id: The ID of parent item., defaults to None
        :type id: Optional[str], optional
        :param user_id: The input for `user_id` is optional. Moving to non-root folder is not allowed when `user_id` is present. Parent folder id should be zero when `user_id` is provided., defaults to None
        :type user_id: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.user_id = user_id


class UpdateFolderByIdSharedLinkAccessField(str, Enum):
    OPEN = 'open'
    COMPANY = 'company'
    COLLABORATORS = 'collaborators'


class UpdateFolderByIdSharedLinkPermissionsField(BaseObject):
    def __init__(self, *, can_download: Optional[bool] = None, **kwargs):
        """
                :param can_download: If the shared link allows for downloading of files.
        This can only be set when `access` is set to
        `open` or `company`., defaults to None
                :type can_download: Optional[bool], optional
        """
        super().__init__(**kwargs)
        self.can_download = can_download


class UpdateFolderByIdSharedLink(BaseObject):
    def __init__(
        self,
        *,
        access: Optional[UpdateFolderByIdSharedLinkAccessField] = None,
        password: Optional[str] = None,
        vanity_name: Optional[str] = None,
        unshared_at: Optional[DateTime] = None,
        permissions: Optional[UpdateFolderByIdSharedLinkPermissionsField] = None,
        **kwargs
    ):
        """
                :param access: The level of access for the shared link. This can be
        restricted to anyone with the link (`open`), only people
        within the company (`company`) and only those who
        have been invited to the folder (`collaborators`).

        If not set, this field defaults to the access level specified
        by the enterprise admin. To create a shared link with this
        default setting pass the `shared_link` object with
        no `access` field, for example `{ "shared_link": {} }`.

        The `company` access level is only available to paid
        accounts., defaults to None
                :type access: Optional[UpdateFolderByIdSharedLinkAccessField], optional
                :param password: The password required to access the shared link. Set the
        password to `null` to remove it.
        Passwords must now be at least eight characters
        long and include a number, upper case letter, or
        a non-numeric or non-alphabetic character.
        A password can only be set when `access` is set to `open`., defaults to None
                :type password: Optional[str], optional
                :param vanity_name: Defines a custom vanity name to use in the shared link URL,
        for example `https://app.box.com/v/my-shared-link`.

        Custom URLs should not be used when sharing sensitive content
        as vanity URLs are a lot easier to guess than regular shared links., defaults to None
                :type vanity_name: Optional[str], optional
                :param unshared_at: The timestamp at which this shared link will
        expire. This field can only be set by
        users with paid accounts., defaults to None
                :type unshared_at: Optional[DateTime], optional
        """
        super().__init__(**kwargs)
        self.access = access
        self.password = password
        self.vanity_name = vanity_name
        self.unshared_at = unshared_at
        self.permissions = permissions


class UpdateFolderByIdFolderUploadEmailAccessField(str, Enum):
    OPEN = 'open'
    COLLABORATORS = 'collaborators'


class UpdateFolderByIdFolderUploadEmail(BaseObject):
    def __init__(
        self,
        *,
        access: Optional[UpdateFolderByIdFolderUploadEmailAccessField] = None,
        **kwargs
    ):
        """
                :param access: When this parameter has been set, users can email files
        to the email address that has been automatically
        created for this folder.

        To create an email address, set this property either when
        creating or updating the folder.

        When set to `collaborators`, only emails from registered email
        addresses for collaborators will be accepted. This includes
        any email aliases a user might have registered.

        When set to `open` it will accept emails from any email
        address., defaults to None
                :type access: Optional[UpdateFolderByIdFolderUploadEmailAccessField], optional
        """
        super().__init__(**kwargs)
        self.access = access


class UpdateFolderByIdCollections(BaseObject):
    def __init__(
        self, *, id: Optional[str] = None, type: Optional[str] = None, **kwargs
    ):
        """
        :param id: The unique identifier for this object., defaults to None
        :type id: Optional[str], optional
        :param type: The type for this object., defaults to None
        :type type: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class GetFolderItemsSort(str, Enum):
    ID = 'id'
    NAME = 'name'
    DATE = 'date'
    SIZE = 'size'


class GetFolderItemsDirection(str, Enum):
    ASC = 'ASC'
    DESC = 'DESC'


class CreateFolderParent(BaseObject):
    def __init__(self, id: str, **kwargs):
        """
        :param id: The ID of parent folder.
        :type id: str
        """
        super().__init__(**kwargs)
        self.id = id


class CreateFolderFolderUploadEmailAccessField(str, Enum):
    OPEN = 'open'
    COLLABORATORS = 'collaborators'


class CreateFolderFolderUploadEmail(BaseObject):
    def __init__(
        self,
        *,
        access: Optional[CreateFolderFolderUploadEmailAccessField] = None,
        **kwargs
    ):
        """
                :param access: When this parameter has been set, users can email files
        to the email address that has been automatically
        created for this folder.

        To create an email address, set this property either when
        creating or updating the folder.

        When set to `collaborators`, only emails from registered email
        addresses for collaborators will be accepted. This includes
        any email aliases a user might have registered.

        When set to `open` it will accept emails from any email
        address., defaults to None
                :type access: Optional[CreateFolderFolderUploadEmailAccessField], optional
        """
        super().__init__(**kwargs)
        self.access = access


class CreateFolderSyncState(str, Enum):
    SYNCED = 'synced'
    NOT_SYNCED = 'not_synced'
    PARTIALLY_SYNCED = 'partially_synced'


class CopyFolderParent(BaseObject):
    def __init__(self, id: str, **kwargs):
        """
        :param id: The ID of parent folder.
        :type id: str
        """
        super().__init__(**kwargs)
        self.id = id


class FoldersManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_folder_by_id(
        self,
        folder_id: str,
        *,
        fields: Optional[List[str]] = None,
        sort: Optional[GetFolderByIdSort] = None,
        direction: Optional[GetFolderByIdDirection] = None,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        if_none_match: Optional[str] = None,
        boxapi: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FolderFull:
        """
                Retrieves details for a folder, including the first 100 entries

                in the folder.


                Passing `sort`, `direction`, `offset`, and `limit`


                parameters in query allows you to manage the


                list of returned


                [folder items](https://developer.box.com/reference/resources/folder--full#param-item-collection).


                To fetch more items within the folder, use the


                [Get items in a folder](https://developer.box.com/reference/get-folders-id-items) endpoint.

                :param folder_id: The unique identifier that represent a folder.

        The ID for any folder can be determined
        by visiting this folder in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/folder/123`
        the `folder_id` is `123`.

        The root folder of a Box account is
        always represented by the ID `0`.
        Example: "12345"
                :type folder_id: str
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested.

        Additionally this field can be used to query any metadata
        applied to the file by specifying the `metadata` field as well
        as the scope and key of the template to retrieve, for example
        `?fields=metadata.enterprise_12345.contractTemplate`., defaults to None
                :type fields: Optional[List[str]], optional
                :param sort: Defines the **second** attribute by which items
        are sorted.

        The folder type affects the way the items
        are sorted:

          * **Standard folder**:
          Items are always sorted by
          their `type` first, with
          folders listed before files,
          and files listed
          before web links.

          * **Root folder**:
          This parameter is not supported
          for marker-based pagination
          on the root folder

          (the folder with an `id` of `0`).

          * **Shared folder with parent path
          to the associated folder visible to
          the collaborator**:
          Items are always sorted by
          their `type` first, with
          folders listed before files,
          and files listed
          before web links., defaults to None
                :type sort: Optional[GetFolderByIdSort], optional
                :param direction: The direction to sort results in. This can be either in alphabetical ascending
        (`ASC`) or descending (`DESC`) order., defaults to None
                :type direction: Optional[GetFolderByIdDirection], optional
                :param offset: The offset of the item at which to begin the response.

        Offset-based pagination is not guaranteed to work reliably for high offset values and may fail for large datasets. In
        those cases, reduce the number of items in the folder (for example, by
        restructuring the folder into smaller subfolders) before retrying the
        request., defaults to None
                :type offset: Optional[int], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param if_none_match: Ensures an item is only returned if it has changed.

        Pass in the item's last observed `etag` value
        into this header and the endpoint will fail
        with a `304 Not Modified` if the item has not
        changed since., defaults to None
                :type if_none_match: Optional[str], optional
                :param boxapi: The URL, and optional password, for the shared link of this item.

        This header can be used to access items that have not been
        explicitly shared with a user.

        Use the format `shared_link=[link]` or if a password is required then
        use `shared_link=[link]&shared_link_password=[password]`.

        This header can be used on the file or folder shared, as well as on any files
        or folders nested within the item., defaults to None
                :type boxapi: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'fields': to_string(fields),
                'sort': to_string(sort),
                'direction': to_string(direction),
                'offset': to_string(offset),
                'limit': to_string(limit),
            }
        )
        headers_map: Dict[str, str] = prepare_params(
            {
                'if-none-match': to_string(if_none_match),
                'boxapi': to_string(boxapi),
                **extra_headers,
            }
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/folders/',
                        to_string(folder_id),
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FolderFull)

    def update_folder_by_id(
        self,
        folder_id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        sync_state: Optional[UpdateFolderByIdSyncState] = None,
        can_non_owners_invite: Optional[bool] = None,
        parent: Optional[UpdateFolderByIdParent] = None,
        shared_link: Optional[UpdateFolderByIdSharedLink] = None,
        folder_upload_email: Union[
            Optional[UpdateFolderByIdFolderUploadEmail], NullValue
        ] = None,
        tags: Optional[List[str]] = None,
        is_collaboration_restricted_to_enterprise: Optional[bool] = None,
        collections: Union[
            Optional[List[UpdateFolderByIdCollections]], NullValue
        ] = None,
        can_non_owners_view_collaborators: Optional[bool] = None,
        fields: Optional[List[str]] = None,
        if_match: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FolderFull:
        r"""
                Updates a folder. This can be also be used to move the folder,

                create shared links, update collaborations, and more.

                :param folder_id: The unique identifier that represent a folder.

        The ID for any folder can be determined
        by visiting this folder in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/folder/123`
        the `folder_id` is `123`.

        The root folder of a Box account is
        always represented by the ID `0`.
        Example: "12345"
                :type folder_id: str
                :param name: The optional new name for this folder.

        The following restrictions to folder names apply: names containing
        non-printable ASCII characters, forward and backward slashes
        (`/`, `\`), names with trailing spaces, and names `.` and `..` are
        not allowed.

        Folder names must be unique within their parent folder. The name check is case-insensitive,
        so a folder named `New Folder` cannot be created in a parent folder that already contains
        a folder named `new folder`., defaults to None
                :type name: Optional[str], optional
                :param description: The optional description of this folder., defaults to None
                :type description: Optional[str], optional
                :param sync_state: Specifies whether a folder should be synced to a
        user's device or not. This is used by Box Sync
        (discontinued) and is not used by Box Drive., defaults to None
                :type sync_state: Optional[UpdateFolderByIdSyncState], optional
                :param can_non_owners_invite: Specifies if users who are not the owner
        of the folder can invite new collaborators to the folder., defaults to None
                :type can_non_owners_invite: Optional[bool], optional
                :param tags: The tags for this item. These tags are shown in
        the Box web app and mobile apps next to an item.

        To add or remove a tag, retrieve the item's current tags,
        modify them, and then update this field.

        There is a limit of 100 tags per item, and 10,000
        unique tags per enterprise., defaults to None
                :type tags: Optional[List[str]], optional
                :param is_collaboration_restricted_to_enterprise: Specifies if new invites to this folder are restricted to users
        within the enterprise. This does not affect existing
        collaborations., defaults to None
                :type is_collaboration_restricted_to_enterprise: Optional[bool], optional
                :param collections: An array of collections to make this folder
        a member of. Currently
        we only support the `favorites` collection.

        To get the ID for a collection, use the
        [List all collections][1] endpoint.

        Passing an empty array `[]` or `null` will remove
        the folder from all collections.

        [1]: https://developer.box.com/reference/get-collections, defaults to None
                :type collections: Union[Optional[List[UpdateFolderByIdCollections]], NullValue], optional
                :param can_non_owners_view_collaborators: Restricts collaborators who are not the owner of
        this folder from viewing other collaborations on
        this folder.

        It also restricts non-owners from inviting new
        collaborators.

        When setting this field to `false`, it is required
        to also set `can_non_owners_invite_collaborators` to
        `false` if it has not already been set., defaults to None
                :type can_non_owners_view_collaborators: Optional[bool], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param if_match: Ensures this item hasn't recently changed before
        making changes.

        Pass in the item's last observed `etag` value
        into this header and the endpoint will fail
        with a `412 Precondition Failed` if it
        has changed since., defaults to None
                :type if_match: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'description': description,
            'sync_state': sync_state,
            'can_non_owners_invite': can_non_owners_invite,
            'parent': parent,
            'shared_link': shared_link,
            'folder_upload_email': folder_upload_email,
            'tags': tags,
            'is_collaboration_restricted_to_enterprise': (
                is_collaboration_restricted_to_enterprise
            ),
            'collections': collections,
            'can_non_owners_view_collaborators': can_non_owners_view_collaborators,
        }
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params(
            {'if-match': to_string(if_match), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/folders/',
                        to_string(folder_id),
                    ]
                ),
                method='PUT',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FolderFull)

    def delete_folder_by_id(
        self,
        folder_id: str,
        *,
        recursive: Optional[bool] = None,
        if_match: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Deletes a folder, either permanently or by moving it to

                the trash.

                :param folder_id: The unique identifier that represent a folder.

        The ID for any folder can be determined
        by visiting this folder in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/folder/123`
        the `folder_id` is `123`.

        The root folder of a Box account is
        always represented by the ID `0`.
        Example: "12345"
                :type folder_id: str
                :param recursive: Delete a folder that is not empty by recursively deleting the
        folder and all of its content., defaults to None
                :type recursive: Optional[bool], optional
                :param if_match: Ensures this item hasn't recently changed before
        making changes.

        Pass in the item's last observed `etag` value
        into this header and the endpoint will fail
        with a `412 Precondition Failed` if it
        has changed since., defaults to None
                :type if_match: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {'recursive': to_string(recursive)}
        )
        headers_map: Dict[str, str] = prepare_params(
            {'if-match': to_string(if_match), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/folders/',
                        to_string(folder_id),
                    ]
                ),
                method='DELETE',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def get_folder_items(
        self,
        folder_id: str,
        *,
        fields: Optional[List[str]] = None,
        usemarker: Optional[bool] = None,
        marker: Optional[str] = None,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        sort: Optional[GetFolderItemsSort] = None,
        direction: Optional[GetFolderItemsDirection] = None,
        boxapi: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> Items:
        """
                Retrieves a page of items in a folder. These items can be files,

                folders, and web links.


                To request more information about the folder itself, like its size,


                use the [Get a folder](https://developer.box.com/reference/get-folders-id) endpoint instead.

                :param folder_id: The unique identifier that represent a folder.

        The ID for any folder can be determined
        by visiting this folder in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/folder/123`
        the `folder_id` is `123`.

        The root folder of a Box account is
        always represented by the ID `0`.
        Example: "12345"
                :type folder_id: str
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested.

        Additionally this field can be used to query any metadata
        applied to the file by specifying the `metadata` field as well
        as the scope and key of the template to retrieve, for example
        `?fields=metadata.enterprise_12345.contractTemplate`., defaults to None
                :type fields: Optional[List[str]], optional
                :param usemarker: Specifies whether to use marker-based pagination instead of
        offset-based pagination. Only one pagination method can
        be used at a time.

        By setting this value to true, the API will return a `marker` field
        that can be passed as a parameter to this endpoint to get the next
        page of the response., defaults to None
                :type usemarker: Optional[bool], optional
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param offset: The offset of the item at which to begin the response.

        Offset-based pagination is not guaranteed to work reliably for high offset values and may fail for large datasets. In
        those cases, use marker-based pagination by setting `usemarker` to `true`., defaults to None
                :type offset: Optional[int], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param sort: Defines the **second** attribute by which items
        are sorted.

        The folder type affects the way the items
        are sorted:

          * **Standard folder**:
          Items are always sorted by
          their `type` first, with
          folders listed before files,
          and files listed
          before web links.

          * **Root folder**:
          This parameter is not supported
          for marker-based pagination
          on the root folder

          (the folder with an `id` of `0`).

          * **Shared folder with parent path
          to the associated folder visible to
          the collaborator**:
          Items are always sorted by
          their `type` first, with
          folders listed before files,
          and files listed
          before web links., defaults to None
                :type sort: Optional[GetFolderItemsSort], optional
                :param direction: The direction to sort results in. This can be either in alphabetical ascending
        (`ASC`) or descending (`DESC`) order., defaults to None
                :type direction: Optional[GetFolderItemsDirection], optional
                :param boxapi: The URL, and optional password, for the shared link of this item.

        This header can be used to access items that have not been
        explicitly shared with a user.

        Use the format `shared_link=[link]` or if a password is required then
        use `shared_link=[link]&shared_link_password=[password]`.

        This header can be used on the file or folder shared, as well as on any files
        or folders nested within the item., defaults to None
                :type boxapi: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'fields': to_string(fields),
                'usemarker': to_string(usemarker),
                'marker': to_string(marker),
                'offset': to_string(offset),
                'limit': to_string(limit),
                'sort': to_string(sort),
                'direction': to_string(direction),
            }
        )
        headers_map: Dict[str, str] = prepare_params(
            {'boxapi': to_string(boxapi), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/folders/',
                        to_string(folder_id),
                        '/items',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, Items)

    def create_folder(
        self,
        name: str,
        parent: CreateFolderParent,
        *,
        folder_upload_email: Optional[CreateFolderFolderUploadEmail] = None,
        sync_state: Optional[CreateFolderSyncState] = None,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FolderFull:
        r"""
                Creates a new empty folder within the specified parent folder.
                :param name: The name for the new folder.

        The following restrictions to folder names apply: names containing
        non-printable ASCII characters, forward and backward slashes
        (`/`, `\`), names with trailing spaces, and names `.` and `..` are
        not allowed.

        Folder names must be unique within their parent folder. The name check is case-insensitive,
        so a folder named `New Folder` cannot be created in a parent folder that already contains
        a folder named `new folder`.
                :type name: str
                :param parent: The parent folder to create the new folder within.
                :type parent: CreateFolderParent
                :param sync_state: Specifies whether a folder should be synced to a
        user's device or not. This is used by Box Sync
        (discontinued) and is not used by Box Drive., defaults to None
                :type sync_state: Optional[CreateFolderSyncState], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'parent': parent,
            'folder_upload_email': folder_upload_email,
            'sync_state': sync_state,
        }
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/folders']),
                method='POST',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FolderFull)

    def copy_folder(
        self,
        folder_id: str,
        parent: CopyFolderParent,
        *,
        name: Optional[str] = None,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FolderFull:
        r"""
                Creates a copy of a folder within a destination folder.

                The original folder will not be changed.

                :param folder_id: The unique identifier of the folder to copy.

        The ID for any folder can be determined
        by visiting this folder in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/folder/123`
        the `folder_id` is `123`.

        The root folder with the ID `0` can not be copied.
        Example: "0"
                :type folder_id: str
                :param parent: The destination folder to copy the folder to.
                :type parent: CopyFolderParent
                :param name: An optional new name for the copied folder.

        There are some restrictions to the file name. Names containing
        non-printable ASCII characters, forward and backward slashes
        (`/`, `\`), as well as names with trailing spaces are
        prohibited.

        Additionally, the names `.` and `..` are
        not allowed either., defaults to None
                :type name: Optional[str], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'name': name, 'parent': parent}
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/folders/',
                        to_string(folder_id),
                        '/copy',
                    ]
                ),
                method='POST',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FolderFull)



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class ZipDownloadNameConflictsTypeField(str, Enum):
    FILE = 'file'
    FOLDER = 'folder'


class ZipDownloadNameConflictsField(BaseObject):
    _discriminator = 'type', {'file', 'folder'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[ZipDownloadNameConflictsTypeField] = None,
        original_name: Optional[str] = None,
        download_name: Optional[str] = None,
        **kwargs
    ):
        """
                :param id: The identifier of the item., defaults to None
                :type id: Optional[str], optional
                :param type: The type of this item., defaults to None
                :type type: Optional[ZipDownloadNameConflictsTypeField], optional
                :param original_name: Box Developer Documentation., defaults to None
                :type original_name: Optional[str], optional
                :param download_name: The new name of this item as it will appear in the
        downloaded `zip` archive., defaults to None
                :type download_name: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.original_name = original_name
        self.download_name = download_name


class ZipDownload(BaseObject):
    def __init__(
        self,
        *,
        download_url: Optional[str] = None,
        status_url: Optional[str] = None,
        expires_at: Optional[DateTime] = None,
        name_conflicts: Optional[List[List[ZipDownloadNameConflictsField]]] = None,
        **kwargs
    ):
        """
                :param download_url: The URL that can be used to download the `zip` archive. A `Get` request to
        this URL will start streaming the items requested. By default, this URL
        is only valid for a few seconds, until the `expires_at` time, unless a
        download is started after which it is valid for the duration of the
        download.

        It is important to note that the domain and path of this URL might change
        between API calls, and therefore it's important to use this URL as-is., defaults to None
                :type download_url: Optional[str], optional
                :param status_url: The URL that can be used to get the status of the `zip` archive being
        downloaded. A `Get` request to this URL will return the number of files
        in the archive as well as the number of items already downloaded or
        skipped. By default, this URL is only valid for a few seconds, until the
        `expires_at` time, unless a download is started after which the URL is
        valid for 12 hours from the start of the download.

        It is important to note that the domain and path of this URL might change
        between API calls, and therefore it's important to use this URL as-is., defaults to None
                :type status_url: Optional[str], optional
                :param expires_at: The time and date when this archive will expire. After this time the
        `status_url` and `download_url` will return an error.

        By default, these URLs are only valid for a few seconds, unless a download
        is started after which the `download_url` is valid for the duration of the
        download, and the `status_url` is valid for 12 hours from the start of the
        download., defaults to None
                :type expires_at: Optional[DateTime], optional
                :param name_conflicts: A list of conflicts that occurred when trying to create the archive. This
        would occur when multiple items have been requested with the
        same name.

        To solve these conflicts, the API will automatically rename an item
        and return a mapping between the original item's name and its new
        name.

        For every conflict, both files will be renamed and therefore this list
        will always be a multiple of 2., defaults to None
                :type name_conflicts: Optional[List[List[ZipDownloadNameConflictsField]]], optional
        """
        super().__init__(**kwargs)
        self.download_url = download_url
        self.status_url = status_url
        self.expires_at = expires_at
        self.name_conflicts = name_conflicts



from enum import Enum

from typing import Optional

from typing import List

from typing import Dict

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.realtime_servers import RealtimeServers

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.events import Events

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import SerializedData

from box_sdk_gen.internal.utils import DateTime

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.box.event_stream import EventStream


class GetEventsStreamType(str, Enum):
    ALL = 'all'
    CHANGES = 'changes'
    SYNC = 'sync'
    ADMIN_LOGS = 'admin_logs'
    ADMIN_LOGS_STREAMING = 'admin_logs_streaming'


class GetEventsEventType(str, Enum):
    ACCESS_GRANTED = 'ACCESS_GRANTED'
    ACCESS_REVOKED = 'ACCESS_REVOKED'
    ADD_DEVICE_ASSOCIATION = 'ADD_DEVICE_ASSOCIATION'
    ADD_LOGIN_ACTIVITY_DEVICE = 'ADD_LOGIN_ACTIVITY_DEVICE'
    ADMIN_LOGIN = 'ADMIN_LOGIN'
    APPLICATION_CREATED = 'APPLICATION_CREATED'
    APPLICATION_PUBLIC_KEY_ADDED = 'APPLICATION_PUBLIC_KEY_ADDED'
    APPLICATION_PUBLIC_KEY_DELETED = 'APPLICATION_PUBLIC_KEY_DELETED'
    CHANGE_ADMIN_ROLE = 'CHANGE_ADMIN_ROLE'
    CHANGE_FOLDER_PERMISSION = 'CHANGE_FOLDER_PERMISSION'
    COLLABORATION_ACCEPT = 'COLLABORATION_ACCEPT'
    COLLABORATION_EXPIRATION = 'COLLABORATION_EXPIRATION'
    COLLABORATION_INVITE = 'COLLABORATION_INVITE'
    COLLABORATION_REMOVE = 'COLLABORATION_REMOVE'
    COLLABORATION_ROLE_CHANGE = 'COLLABORATION_ROLE_CHANGE'
    COMMENT_CREATE = 'COMMENT_CREATE'
    COMMENT_DELETE = 'COMMENT_DELETE'
    CONTENT_WORKFLOW_ABNORMAL_DOWNLOAD_ACTIVITY = (
        'CONTENT_WORKFLOW_ABNORMAL_DOWNLOAD_ACTIVITY'
    )
    CONTENT_WORKFLOW_AUTOMATION_ADD = 'CONTENT_WORKFLOW_AUTOMATION_ADD'
    CONTENT_WORKFLOW_AUTOMATION_DELETE = 'CONTENT_WORKFLOW_AUTOMATION_DELETE'
    CONTENT_WORKFLOW_POLICY_ADD = 'CONTENT_WORKFLOW_POLICY_ADD'
    CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION = (
        'CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION'
    )
    CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION = (
        'CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION'
    )
    COPY = 'COPY'
    DATA_RETENTION_CREATE_RETENTION = 'DATA_RETENTION_CREATE_RETENTION'
    DATA_RETENTION_REMOVE_RETENTION = 'DATA_RETENTION_REMOVE_RETENTION'
    DELETE = 'DELETE'
    DELETE_USER = 'DELETE_USER'
    DEVICE_TRUST_CHECK_FAILED = 'DEVICE_TRUST_CHECK_FAILED'
    DOWNLOAD = 'DOWNLOAD'
    EDIT = 'EDIT'
    EDIT_USER = 'EDIT_USER'
    EMAIL_ALIAS_CONFIRM = 'EMAIL_ALIAS_CONFIRM'
    EMAIL_ALIAS_REMOVE = 'EMAIL_ALIAS_REMOVE'
    ENTERPRISE_APP_AUTHORIZATION_UPDATE = 'ENTERPRISE_APP_AUTHORIZATION_UPDATE'
    EXTERNAL_COLLAB_SECURITY_SETTINGS = 'EXTERNAL_COLLAB_SECURITY_SETTINGS'
    FAILED_LOGIN = 'FAILED_LOGIN'
    FILE_MARKED_MALICIOUS = 'FILE_MARKED_MALICIOUS'
    FILE_WATERMARKED_DOWNLOAD = 'FILE_WATERMARKED_DOWNLOAD'
    GROUP_ADD_ITEM = 'GROUP_ADD_ITEM'
    GROUP_ADD_USER = 'GROUP_ADD_USER'
    GROUP_CREATION = 'GROUP_CREATION'
    GROUP_DELETION = 'GROUP_DELETION'
    GROUP_EDITED = 'GROUP_EDITED'
    GROUP_REMOVE_ITEM = 'GROUP_REMOVE_ITEM'
    GROUP_REMOVE_USER = 'GROUP_REMOVE_USER'
    ITEM_EMAIL_SEND = 'ITEM_EMAIL_SEND'
    ITEM_MODIFY = 'ITEM_MODIFY'
    ITEM_OPEN = 'ITEM_OPEN'
    ITEM_SHARED_UPDATE = 'ITEM_SHARED_UPDATE'
    ITEM_SYNC = 'ITEM_SYNC'
    ITEM_UNSYNC = 'ITEM_UNSYNC'
    LEGAL_HOLD_ASSIGNMENT_CREATE = 'LEGAL_HOLD_ASSIGNMENT_CREATE'
    LEGAL_HOLD_ASSIGNMENT_DELETE = 'LEGAL_HOLD_ASSIGNMENT_DELETE'
    LEGAL_HOLD_POLICY_CREATE = 'LEGAL_HOLD_POLICY_CREATE'
    LEGAL_HOLD_POLICY_DELETE = 'LEGAL_HOLD_POLICY_DELETE'
    LEGAL_HOLD_POLICY_UPDATE = 'LEGAL_HOLD_POLICY_UPDATE'
    LOCK = 'LOCK'
    LOGIN = 'LOGIN'
    METADATA_INSTANCE_CREATE = 'METADATA_INSTANCE_CREATE'
    METADATA_INSTANCE_DELETE = 'METADATA_INSTANCE_DELETE'
    METADATA_INSTANCE_UPDATE = 'METADATA_INSTANCE_UPDATE'
    METADATA_TEMPLATE_CREATE = 'METADATA_TEMPLATE_CREATE'
    METADATA_TEMPLATE_DELETE = 'METADATA_TEMPLATE_DELETE'
    METADATA_TEMPLATE_UPDATE = 'METADATA_TEMPLATE_UPDATE'
    MOVE = 'MOVE'
    NEW_USER = 'NEW_USER'
    OAUTH2_ACCESS_TOKEN_REVOKE = 'OAUTH2_ACCESS_TOKEN_REVOKE'
    PREVIEW = 'PREVIEW'
    REMOVE_DEVICE_ASSOCIATION = 'REMOVE_DEVICE_ASSOCIATION'
    REMOVE_LOGIN_ACTIVITY_DEVICE = 'REMOVE_LOGIN_ACTIVITY_DEVICE'
    RENAME = 'RENAME'
    RETENTION_POLICY_ASSIGNMENT_ADD = 'RETENTION_POLICY_ASSIGNMENT_ADD'
    SHARE = 'SHARE'
    SHARED_LINK_SEND = 'SHARED_LINK_SEND'
    SHARE_EXPIRATION = 'SHARE_EXPIRATION'
    SHIELD_ALERT = 'SHIELD_ALERT'
    SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED = 'SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED'
    SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED_MISSING_JUSTIFICATION = (
        'SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED_MISSING_JUSTIFICATION'
    )
    SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED = 'SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED'
    SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED_MISSING_JUSTIFICATION = (
        'SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED_MISSING_JUSTIFICATION'
    )
    SHIELD_JUSTIFICATION_APPROVAL = 'SHIELD_JUSTIFICATION_APPROVAL'
    SHIELD_SHARED_LINK_ACCESS_BLOCKED = 'SHIELD_SHARED_LINK_ACCESS_BLOCKED'
    SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_CREATE = (
        'SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_CREATE'
    )
    SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_UPDATE = (
        'SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_UPDATE'
    )
    SIGN_DOCUMENT_ASSIGNED = 'SIGN_DOCUMENT_ASSIGNED'
    SIGN_DOCUMENT_CANCELLED = 'SIGN_DOCUMENT_CANCELLED'
    SIGN_DOCUMENT_COMPLETED = 'SIGN_DOCUMENT_COMPLETED'
    SIGN_DOCUMENT_CONVERTED = 'SIGN_DOCUMENT_CONVERTED'
    SIGN_DOCUMENT_CREATED = 'SIGN_DOCUMENT_CREATED'
    SIGN_DOCUMENT_DECLINED = 'SIGN_DOCUMENT_DECLINED'
    SIGN_DOCUMENT_EXPIRED = 'SIGN_DOCUMENT_EXPIRED'
    SIGN_DOCUMENT_SIGNED = 'SIGN_DOCUMENT_SIGNED'
    SIGN_DOCUMENT_VIEWED_BY_SIGNED = 'SIGN_DOCUMENT_VIEWED_BY_SIGNED'
    SIGNER_DOWNLOADED = 'SIGNER_DOWNLOADED'
    SIGNER_FORWARDED = 'SIGNER_FORWARDED'
    STORAGE_EXPIRATION = 'STORAGE_EXPIRATION'
    TASK_ASSIGNMENT_CREATE = 'TASK_ASSIGNMENT_CREATE'
    TASK_ASSIGNMENT_DELETE = 'TASK_ASSIGNMENT_DELETE'
    TASK_ASSIGNMENT_UPDATE = 'TASK_ASSIGNMENT_UPDATE'
    TASK_CREATE = 'TASK_CREATE'
    TASK_UPDATE = 'TASK_UPDATE'
    TERMS_OF_SERVICE_ACCEPT = 'TERMS_OF_SERVICE_ACCEPT'
    TERMS_OF_SERVICE_REJECT = 'TERMS_OF_SERVICE_REJECT'
    UNDELETE = 'UNDELETE'
    UNLOCK = 'UNLOCK'
    UNSHARE = 'UNSHARE'
    UPDATE_COLLABORATION_EXPIRATION = 'UPDATE_COLLABORATION_EXPIRATION'
    UPDATE_SHARE_EXPIRATION = 'UPDATE_SHARE_EXPIRATION'
    UPLOAD = 'UPLOAD'
    USER_AUTHENTICATE_OAUTH2_ACCESS_TOKEN_CREATE = (
        'USER_AUTHENTICATE_OAUTH2_ACCESS_TOKEN_CREATE'
    )
    WATERMARK_LABEL_CREATE = 'WATERMARK_LABEL_CREATE'
    WATERMARK_LABEL_DELETE = 'WATERMARK_LABEL_DELETE'


class GetEventStreamQueryParamsStreamTypeField(str, Enum):
    ALL = 'all'
    CHANGES = 'changes'
    SYNC = 'sync'
    ADMIN_LOGS = 'admin_logs'
    ADMIN_LOGS_STREAMING = 'admin_logs_streaming'


class GetEventStreamQueryParamsEventTypeField(str, Enum):
    ACCESS_GRANTED = 'ACCESS_GRANTED'
    ACCESS_REVOKED = 'ACCESS_REVOKED'
    ADD_DEVICE_ASSOCIATION = 'ADD_DEVICE_ASSOCIATION'
    ADD_LOGIN_ACTIVITY_DEVICE = 'ADD_LOGIN_ACTIVITY_DEVICE'
    ADMIN_LOGIN = 'ADMIN_LOGIN'
    APPLICATION_CREATED = 'APPLICATION_CREATED'
    APPLICATION_PUBLIC_KEY_ADDED = 'APPLICATION_PUBLIC_KEY_ADDED'
    APPLICATION_PUBLIC_KEY_DELETED = 'APPLICATION_PUBLIC_KEY_DELETED'
    CHANGE_ADMIN_ROLE = 'CHANGE_ADMIN_ROLE'
    CHANGE_FOLDER_PERMISSION = 'CHANGE_FOLDER_PERMISSION'
    COLLABORATION_ACCEPT = 'COLLABORATION_ACCEPT'
    COLLABORATION_EXPIRATION = 'COLLABORATION_EXPIRATION'
    COLLABORATION_INVITE = 'COLLABORATION_INVITE'
    COLLABORATION_REMOVE = 'COLLABORATION_REMOVE'
    COLLABORATION_ROLE_CHANGE = 'COLLABORATION_ROLE_CHANGE'
    COMMENT_CREATE = 'COMMENT_CREATE'
    COMMENT_DELETE = 'COMMENT_DELETE'
    CONTENT_WORKFLOW_ABNORMAL_DOWNLOAD_ACTIVITY = (
        'CONTENT_WORKFLOW_ABNORMAL_DOWNLOAD_ACTIVITY'
    )
    CONTENT_WORKFLOW_AUTOMATION_ADD = 'CONTENT_WORKFLOW_AUTOMATION_ADD'
    CONTENT_WORKFLOW_AUTOMATION_DELETE = 'CONTENT_WORKFLOW_AUTOMATION_DELETE'
    CONTENT_WORKFLOW_POLICY_ADD = 'CONTENT_WORKFLOW_POLICY_ADD'
    CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION = (
        'CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION'
    )
    CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION = (
        'CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION'
    )
    COPY = 'COPY'
    DATA_RETENTION_CREATE_RETENTION = 'DATA_RETENTION_CREATE_RETENTION'
    DATA_RETENTION_REMOVE_RETENTION = 'DATA_RETENTION_REMOVE_RETENTION'
    DELETE = 'DELETE'
    DELETE_USER = 'DELETE_USER'
    DEVICE_TRUST_CHECK_FAILED = 'DEVICE_TRUST_CHECK_FAILED'
    DOWNLOAD = 'DOWNLOAD'
    EDIT = 'EDIT'
    EDIT_USER = 'EDIT_USER'
    EMAIL_ALIAS_CONFIRM = 'EMAIL_ALIAS_CONFIRM'
    EMAIL_ALIAS_REMOVE = 'EMAIL_ALIAS_REMOVE'
    ENTERPRISE_APP_AUTHORIZATION_UPDATE = 'ENTERPRISE_APP_AUTHORIZATION_UPDATE'
    EXTERNAL_COLLAB_SECURITY_SETTINGS = 'EXTERNAL_COLLAB_SECURITY_SETTINGS'
    FAILED_LOGIN = 'FAILED_LOGIN'
    FILE_MARKED_MALICIOUS = 'FILE_MARKED_MALICIOUS'
    FILE_WATERMARKED_DOWNLOAD = 'FILE_WATERMARKED_DOWNLOAD'
    GROUP_ADD_ITEM = 'GROUP_ADD_ITEM'
    GROUP_ADD_USER = 'GROUP_ADD_USER'
    GROUP_CREATION = 'GROUP_CREATION'
    GROUP_DELETION = 'GROUP_DELETION'
    GROUP_EDITED = 'GROUP_EDITED'
    GROUP_REMOVE_ITEM = 'GROUP_REMOVE_ITEM'
    GROUP_REMOVE_USER = 'GROUP_REMOVE_USER'
    ITEM_EMAIL_SEND = 'ITEM_EMAIL_SEND'
    ITEM_MODIFY = 'ITEM_MODIFY'
    ITEM_OPEN = 'ITEM_OPEN'
    ITEM_SHARED_UPDATE = 'ITEM_SHARED_UPDATE'
    ITEM_SYNC = 'ITEM_SYNC'
    ITEM_UNSYNC = 'ITEM_UNSYNC'
    LEGAL_HOLD_ASSIGNMENT_CREATE = 'LEGAL_HOLD_ASSIGNMENT_CREATE'
    LEGAL_HOLD_ASSIGNMENT_DELETE = 'LEGAL_HOLD_ASSIGNMENT_DELETE'
    LEGAL_HOLD_POLICY_CREATE = 'LEGAL_HOLD_POLICY_CREATE'
    LEGAL_HOLD_POLICY_DELETE = 'LEGAL_HOLD_POLICY_DELETE'
    LEGAL_HOLD_POLICY_UPDATE = 'LEGAL_HOLD_POLICY_UPDATE'
    LOCK = 'LOCK'
    LOGIN = 'LOGIN'
    METADATA_INSTANCE_CREATE = 'METADATA_INSTANCE_CREATE'
    METADATA_INSTANCE_DELETE = 'METADATA_INSTANCE_DELETE'
    METADATA_INSTANCE_UPDATE = 'METADATA_INSTANCE_UPDATE'
    METADATA_TEMPLATE_CREATE = 'METADATA_TEMPLATE_CREATE'
    METADATA_TEMPLATE_DELETE = 'METADATA_TEMPLATE_DELETE'
    METADATA_TEMPLATE_UPDATE = 'METADATA_TEMPLATE_UPDATE'
    MOVE = 'MOVE'
    NEW_USER = 'NEW_USER'
    OAUTH2_ACCESS_TOKEN_REVOKE = 'OAUTH2_ACCESS_TOKEN_REVOKE'
    PREVIEW = 'PREVIEW'
    REMOVE_DEVICE_ASSOCIATION = 'REMOVE_DEVICE_ASSOCIATION'
    REMOVE_LOGIN_ACTIVITY_DEVICE = 'REMOVE_LOGIN_ACTIVITY_DEVICE'
    RENAME = 'RENAME'
    RETENTION_POLICY_ASSIGNMENT_ADD = 'RETENTION_POLICY_ASSIGNMENT_ADD'
    SHARE = 'SHARE'
    SHARED_LINK_SEND = 'SHARED_LINK_SEND'
    SHARE_EXPIRATION = 'SHARE_EXPIRATION'
    SHIELD_ALERT = 'SHIELD_ALERT'
    SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED = 'SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED'
    SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED_MISSING_JUSTIFICATION = (
        'SHIELD_EXTERNAL_COLLAB_ACCESS_BLOCKED_MISSING_JUSTIFICATION'
    )
    SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED = 'SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED'
    SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED_MISSING_JUSTIFICATION = (
        'SHIELD_EXTERNAL_COLLAB_INVITE_BLOCKED_MISSING_JUSTIFICATION'
    )
    SHIELD_JUSTIFICATION_APPROVAL = 'SHIELD_JUSTIFICATION_APPROVAL'
    SHIELD_SHARED_LINK_ACCESS_BLOCKED = 'SHIELD_SHARED_LINK_ACCESS_BLOCKED'
    SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_CREATE = (
        'SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_CREATE'
    )
    SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_UPDATE = (
        'SHIELD_SHARED_LINK_STATUS_RESTRICTED_ON_UPDATE'
    )
    SIGN_DOCUMENT_ASSIGNED = 'SIGN_DOCUMENT_ASSIGNED'
    SIGN_DOCUMENT_CANCELLED = 'SIGN_DOCUMENT_CANCELLED'
    SIGN_DOCUMENT_COMPLETED = 'SIGN_DOCUMENT_COMPLETED'
    SIGN_DOCUMENT_CONVERTED = 'SIGN_DOCUMENT_CONVERTED'
    SIGN_DOCUMENT_CREATED = 'SIGN_DOCUMENT_CREATED'
    SIGN_DOCUMENT_DECLINED = 'SIGN_DOCUMENT_DECLINED'
    SIGN_DOCUMENT_EXPIRED = 'SIGN_DOCUMENT_EXPIRED'
    SIGN_DOCUMENT_SIGNED = 'SIGN_DOCUMENT_SIGNED'
    SIGN_DOCUMENT_VIEWED_BY_SIGNED = 'SIGN_DOCUMENT_VIEWED_BY_SIGNED'
    SIGNER_DOWNLOADED = 'SIGNER_DOWNLOADED'
    SIGNER_FORWARDED = 'SIGNER_FORWARDED'
    STORAGE_EXPIRATION = 'STORAGE_EXPIRATION'
    TASK_ASSIGNMENT_CREATE = 'TASK_ASSIGNMENT_CREATE'
    TASK_ASSIGNMENT_DELETE = 'TASK_ASSIGNMENT_DELETE'
    TASK_ASSIGNMENT_UPDATE = 'TASK_ASSIGNMENT_UPDATE'
    TASK_CREATE = 'TASK_CREATE'
    TASK_UPDATE = 'TASK_UPDATE'
    TERMS_OF_SERVICE_ACCEPT = 'TERMS_OF_SERVICE_ACCEPT'
    TERMS_OF_SERVICE_REJECT = 'TERMS_OF_SERVICE_REJECT'
    UNDELETE = 'UNDELETE'
    UNLOCK = 'UNLOCK'
    UNSHARE = 'UNSHARE'
    UPDATE_COLLABORATION_EXPIRATION = 'UPDATE_COLLABORATION_EXPIRATION'
    UPDATE_SHARE_EXPIRATION = 'UPDATE_SHARE_EXPIRATION'
    UPLOAD = 'UPLOAD'
    USER_AUTHENTICATE_OAUTH2_ACCESS_TOKEN_CREATE = (
        'USER_AUTHENTICATE_OAUTH2_ACCESS_TOKEN_CREATE'
    )
    WATERMARK_LABEL_CREATE = 'WATERMARK_LABEL_CREATE'
    WATERMARK_LABEL_DELETE = 'WATERMARK_LABEL_DELETE'


class GetEventStreamQueryParams:
    def __init__(
        self,
        *,
        stream_type: Optional[GetEventStreamQueryParamsStreamTypeField] = None,
        stream_position: Optional[str] = None,
        limit: Optional[int] = None,
        event_type: Optional[List[GetEventStreamQueryParamsEventTypeField]] = None,
        created_after: Optional[DateTime] = None,
        created_before: Optional[DateTime] = None
    ):
        """
                :param stream_type: Defines the type of events that are returned

        * `all` returns everything for a user and is the default
        * `changes` returns events that may cause file tree changes
          such as file updates or collaborations.
        * `sync` is similar to `changes` but only applies to synced folders
        * `admin_logs` returns all events for an entire enterprise and
          requires the user making the API call to have admin permissions. This
          stream type is for programmatically pulling from a 1 year history of
          events across all users within the enterprise and within a
          `created_after` and `created_before` time frame. The complete history
          of events will be returned in chronological order based on the event
          time, but latency will be much higher than `admin_logs_streaming`.
        * `admin_logs_streaming` returns all events for an entire enterprise and
          requires the user making the API call to have admin permissions. This
          stream type is for polling for recent events across all users within
          the enterprise. Latency will be much lower than `admin_logs`, but
          events will not be returned in chronological order and may
          contain duplicates., defaults to None
                :type stream_type: Optional[GetEventStreamQueryParamsStreamTypeField], optional
                :param stream_position: The location in the event stream to start receiving events from.

        * `now` will return an empty list events and
        the latest stream position for initialization.
        * `0` or `null` will return all events., defaults to None
                :type stream_position: Optional[str], optional
                :param limit: Limits the number of events returned.

        Note: Sometimes, the events less than the limit requested can be returned
        even when there may be more events remaining. This is primarily done in
        the case where a number of events have already been retrieved and these
        retrieved events are returned rather than delaying for an unknown amount
        of time to see if there are any more results., defaults to None
                :type limit: Optional[int], optional
                :param event_type: A comma-separated list of events to filter by. This can only be used when
        requesting the events with a `stream_type` of `admin_logs` or
        `adming_logs_streaming`. For any other `stream_type` this value will be
        ignored., defaults to None
                :type event_type: Optional[List[GetEventStreamQueryParamsEventTypeField]], optional
                :param created_after: The lower bound date and time to return events for. This can only be used
        when requesting the events with a `stream_type` of `admin_logs`. For any
        other `stream_type` this value will be ignored., defaults to None
                :type created_after: Optional[DateTime], optional
                :param created_before: The upper bound date and time to return events for. This can only be used
        when requesting the events with a `stream_type` of `admin_logs`. For any
        other `stream_type` this value will be ignored., defaults to None
                :type created_before: Optional[DateTime], optional
        """
        self.stream_type = stream_type
        self.stream_position = stream_position
        self.limit = limit
        self.event_type = event_type
        self.created_after = created_after
        self.created_before = created_before


class GetEventStreamHeaders:
    def __init__(self, *, extra_headers: Optional[Dict[str, Optional[str]]] = None):
        """
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        self.extra_headers = extra_headers


class EventsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_events_with_long_polling(
        self, *, extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> RealtimeServers:
        """
        Returns a list of real-time servers that can be used for long-polling updates

        to the [event stream](https://developer.box.com/reference/get-events).


        Long polling is the concept where a HTTP request is kept open until the


        server sends a response, then repeating the process over and over to receive


        updated responses.


        Long polling the event stream can only be used for user events, not for


        enterprise events.


        To use long polling, first use this endpoint to retrieve a list of long poll


        URLs. Next, make a long poll request to any of the provided URLs.


        When an event occurs in monitored account a response with the value


        `new_change` will be sent. The response contains no other details as


        it only serves as a prompt to take further action such as sending a


        request to the [events endpoint](https://developer.box.com/reference/get-events) with the last known


        `stream_position`.


        After the server sends this response it closes the connection. You must now


        repeat the long poll process to begin listening for events again.


        If no events occur for a while and the connection times out you will


        receive a response with the value `reconnect`. When you receive this response


        you’ll make another call to this endpoint to restart the process.


        If you receive no events in `retry_timeout` seconds then you will need to


        make another request to the real-time server (one of the URLs in the response


        for this endpoint). This might be necessary due to network errors.


        Finally, if you receive a `max_retries` error when making a request to the


        real-time server, you should start over by making a call to this endpoint


        first.

        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/events']),
                method='OPTIONS',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, RealtimeServers)

    def get_events(
        self,
        *,
        stream_type: Optional[GetEventsStreamType] = None,
        stream_position: Optional[str] = None,
        limit: Optional[int] = None,
        event_type: Optional[List[GetEventsEventType]] = None,
        created_after: Optional[DateTime] = None,
        created_before: Optional[DateTime] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> Events:
        """
                Returns up to a year of past events for a given user

                or for the entire enterprise.


                By default this returns events for the authenticated user. To retrieve events


                for the entire enterprise, set the `stream_type` to `admin_logs_streaming`


                for live monitoring of new events, or `admin_logs` for querying across


                historical events. The user making the API call will


                need to have admin privileges, and the application will need to have the


                scope `manage enterprise properties` checked.

                :param stream_type: Defines the type of events that are returned

        * `all` returns everything for a user and is the default
        * `changes` returns events that may cause file tree changes
          such as file updates or collaborations.
        * `sync` is similar to `changes` but only applies to synced folders
        * `admin_logs` returns all events for an entire enterprise and
          requires the user making the API call to have admin permissions. This
          stream type is for programmatically pulling from a 1 year history of
          events across all users within the enterprise and within a
          `created_after` and `created_before` time frame. The complete history
          of events will be returned in chronological order based on the event
          time, but latency will be much higher than `admin_logs_streaming`.
        * `admin_logs_streaming` returns all events for an entire enterprise and
          requires the user making the API call to have admin permissions. This
          stream type is for polling for recent events across all users within
          the enterprise. Latency will be much lower than `admin_logs`, but
          events will not be returned in chronological order and may
          contain duplicates., defaults to None
                :type stream_type: Optional[GetEventsStreamType], optional
                :param stream_position: The location in the event stream to start receiving events from.

        * `now` will return an empty list events and
        the latest stream position for initialization.
        * `0` or `null` will return all events., defaults to None
                :type stream_position: Optional[str], optional
                :param limit: Limits the number of events returned.

        Note: Sometimes, the events less than the limit requested can be returned
        even when there may be more events remaining. This is primarily done in
        the case where a number of events have already been retrieved and these
        retrieved events are returned rather than delaying for an unknown amount
        of time to see if there are any more results., defaults to None
                :type limit: Optional[int], optional
                :param event_type: A comma-separated list of events to filter by. This can only be used when
        requesting the events with a `stream_type` of `admin_logs` or
        `adming_logs_streaming`. For any other `stream_type` this value will be
        ignored., defaults to None
                :type event_type: Optional[List[GetEventsEventType]], optional
                :param created_after: The lower bound date and time to return events for. This can only be used
        when requesting the events with a `stream_type` of `admin_logs`. For any
        other `stream_type` this value will be ignored., defaults to None
                :type created_after: Optional[DateTime], optional
                :param created_before: The upper bound date and time to return events for. This can only be used
        when requesting the events with a `stream_type` of `admin_logs`. For any
        other `stream_type` this value will be ignored., defaults to None
                :type created_before: Optional[DateTime], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'stream_type': to_string(stream_type),
                'stream_position': to_string(stream_position),
                'limit': to_string(limit),
                'event_type': to_string(event_type),
                'created_after': to_string(created_after),
                'created_before': to_string(created_before),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/events']),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, Events)

    def get_event_stream(
        self,
        *,
        query_params: GetEventStreamQueryParams = None,
        headers: GetEventStreamHeaders = None
    ) -> EventStream:
        """
        Get an event stream for the Box API
        :param query_params: Query parameters of getEvents method, defaults to None
        :type query_params: GetEventStreamQueryParams, optional
        :param headers: Headers of getEvents method, defaults to None
        :type headers: GetEventStreamHeaders, optional
        """
        if query_params is None:
            query_params = GetEventStreamQueryParams()
        if headers is None:
            headers = GetEventStreamHeaders()
        return EventStream(
            events_manager=self, query_params=query_params, headers_input=headers
        )



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.user_mini import UserMini

from box_sdk_gen.schemas.group_mini import GroupMini

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class GroupMembershipTypeField(str, Enum):
    GROUP_MEMBERSHIP = 'group_membership'


class GroupMembershipRoleField(str, Enum):
    MEMBER = 'member'
    ADMIN = 'admin'


class GroupMembership(BaseObject):
    _discriminator = 'type', {'group_membership'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[GroupMembershipTypeField] = None,
        user: Optional[UserMini] = None,
        group: Optional[GroupMini] = None,
        role: Optional[GroupMembershipRoleField] = None,
        created_at: Optional[DateTime] = None,
        modified_at: Optional[DateTime] = None,
        **kwargs
    ):
        """
        :param id: The unique identifier for this group membership., defaults to None
        :type id: Optional[str], optional
        :param type: The value will always be `group_membership`., defaults to None
        :type type: Optional[GroupMembershipTypeField], optional
        :param role: The role of the user in the group., defaults to None
        :type role: Optional[GroupMembershipRoleField], optional
        :param created_at: The time this membership was created., defaults to None
        :type created_at: Optional[DateTime], optional
        :param modified_at: The time this membership was last modified., defaults to None
        :type modified_at: Optional[DateTime], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.user = user
        self.group = group
        self.role = role
        self.created_at = created_at
        self.modified_at = modified_at



from typing import Optional

from typing import List

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.v2025_r0.hub_item_operation_v2025_r0 import (
    HubItemOperationV2025R0,
)

from box_sdk_gen.box.errors import BoxSDKError


class HubItemsManageRequestV2025R0(BaseObject):
    def __init__(
        self, *, operations: Optional[List[HubItemOperationV2025R0]] = None, **kwargs
    ):
        """
        :param operations: List of operations to perform on Box Hub items., defaults to None
        :type operations: Optional[List[HubItemOperationV2025R0]], optional
        """
        super().__init__(**kwargs)
        self.operations = operations



from box_sdk_gen.managers.authorization import *

from box_sdk_gen.managers.files import *

from box_sdk_gen.managers.trashed_files import *

from box_sdk_gen.managers.app_item_associations import *

from box_sdk_gen.managers.downloads import *

from box_sdk_gen.managers.uploads import *

from box_sdk_gen.managers.chunked_uploads import *

from box_sdk_gen.managers.list_collaborations import *

from box_sdk_gen.managers.comments import *

from box_sdk_gen.managers.tasks import *

from box_sdk_gen.managers.file_versions import *

from box_sdk_gen.managers.file_metadata import *

from box_sdk_gen.managers.file_classifications import *

from box_sdk_gen.managers.skills import *

from box_sdk_gen.managers.file_watermarks import *

from box_sdk_gen.managers.file_requests import *

from box_sdk_gen.managers.folders import *

from box_sdk_gen.managers.trashed_folders import *

from box_sdk_gen.managers.folder_metadata import *

from box_sdk_gen.managers.folder_classifications import *

from box_sdk_gen.managers.trashed_items import *

from box_sdk_gen.managers.folder_watermarks import *

from box_sdk_gen.managers.folder_locks import *

from box_sdk_gen.managers.metadata_templates import *

from box_sdk_gen.managers.classifications import *

from box_sdk_gen.managers.metadata_cascade_policies import *

from box_sdk_gen.managers.search import *

from box_sdk_gen.managers.user_collaborations import *

from box_sdk_gen.managers.task_assignments import *

from box_sdk_gen.managers.shared_links_files import *

from box_sdk_gen.managers.shared_links_folders import *

from box_sdk_gen.managers.web_links import *

from box_sdk_gen.managers.trashed_web_links import *

from box_sdk_gen.managers.shared_links_web_links import *

from box_sdk_gen.managers.shared_links_app_items import *

from box_sdk_gen.managers.users import *

from box_sdk_gen.managers.session_termination import *

from box_sdk_gen.managers.avatars import *

from box_sdk_gen.managers.transfer import *

from box_sdk_gen.managers.email_aliases import *

from box_sdk_gen.managers.memberships import *

from box_sdk_gen.managers.invites import *

from box_sdk_gen.managers.groups import *

from box_sdk_gen.managers.webhooks import *

from box_sdk_gen.managers.events import *

from box_sdk_gen.managers.collections import *

from box_sdk_gen.managers.recent_items import *

from box_sdk_gen.managers.retention_policies import *

from box_sdk_gen.managers.retention_policy_assignments import *

from box_sdk_gen.managers.legal_hold_policies import *

from box_sdk_gen.managers.legal_hold_policy_assignments import *

from box_sdk_gen.managers.file_version_retentions import *

from box_sdk_gen.managers.file_version_legal_holds import *

from box_sdk_gen.managers.shield_information_barriers import *

from box_sdk_gen.managers.shield_information_barrier_reports import *

from box_sdk_gen.managers.shield_information_barrier_segments import *

from box_sdk_gen.managers.shield_information_barrier_segment_members import *

from box_sdk_gen.managers.shield_information_barrier_segment_restrictions import *

from box_sdk_gen.managers.device_pinners import *

from box_sdk_gen.managers.terms_of_services import *

from box_sdk_gen.managers.terms_of_service_user_statuses import *

from box_sdk_gen.managers.collaboration_allowlist_entries import *

from box_sdk_gen.managers.collaboration_allowlist_exempt_targets import *

from box_sdk_gen.managers.storage_policies import *

from box_sdk_gen.managers.storage_policy_assignments import *

from box_sdk_gen.managers.zip_downloads import *

from box_sdk_gen.managers.sign_requests import *

from box_sdk_gen.managers.workflows import *

from box_sdk_gen.managers.sign_templates import *

from box_sdk_gen.managers.integration_mappings import *

from box_sdk_gen.managers.ai import *

from box_sdk_gen.managers.ai_studio import *

from box_sdk_gen.managers.metadata_taxonomies import *

from box_sdk_gen.managers.docgen_template import *

from box_sdk_gen.managers.docgen import *

from box_sdk_gen.managers.enterprise_configurations import *

from box_sdk_gen.managers.hubs import *

from box_sdk_gen.managers.hub_collaborations import *

from box_sdk_gen.managers.hub_items import *

from box_sdk_gen.managers.shield_lists import *

from box_sdk_gen.managers.archives import *

from box_sdk_gen.managers.external_users import *



import shelve
from abc import abstractmethod
from typing import Optional

from ..schemas.access_token import AccessToken


class TokenStorage:
    @abstractmethod
    def store(self, token: AccessToken) -> None:
        pass

    @abstractmethod
    def get(self) -> Optional[AccessToken]:
        pass

    @abstractmethod
    def clear(self) -> None:
        pass


class InMemoryTokenStorage(TokenStorage):
    def __init__(self, token: Optional[AccessToken] = None):
        self._token = token

    def store(self, token: AccessToken) -> None:
        self._token = token

    def get(self) -> Optional[AccessToken]:
        return self._token

    def clear(self) -> None:
        self._token = None


class FileTokenStorage(TokenStorage):
    def __init__(self, filename: str = 'token_storage'):
        self.filename = filename

    def store(self, token: AccessToken) -> None:
        with shelve.open(self.filename) as file:
            file['token'] = token

    def get(self) -> Optional[AccessToken]:
        with shelve.open(self.filename) as file:
            return file.get('token', None)

    def clear(self) -> None:
        with shelve.open(self.filename) as file:
            if 'token' in file:
                del file['token']


class FileWithInMemoryCacheTokenStorage(TokenStorage):
    def __init__(self, filename: str = 'token_storage'):
        self.filename = filename
        self.cached_token: Optional[AccessToken] = None

    def store(self, token: AccessToken) -> None:
        with shelve.open(self.filename) as file:
            file['token'] = token
        self.cached_token = token

    def get(self) -> Optional[AccessToken]:
        if self.cached_token is None:
            with shelve.open(self.filename) as file:
                self.cached_token = file.get('token', None)
        return self.cached_token

    def clear(self) -> None:
        with shelve.open(self.filename) as file:
            if 'token' in file:
                del file['token']
        self.cached_token = None



from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.v2025_r0.shield_list_content_country_v2025_r0 import (
    ShieldListContentCountryV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_domain_v2025_r0 import (
    ShieldListContentDomainV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_email_v2025_r0 import (
    ShieldListContentEmailV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_ip_v2025_r0 import (
    ShieldListContentIpV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_request_v2025_r0 import (
    ShieldListContentRequestV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_lists_v2025_r0 import ShieldListsV2025R0

from box_sdk_gen.parameters.v2025_r0.box_version_header_v2025_r0 import (
    BoxVersionHeaderV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_v2025_r0 import ShieldListV2025R0

from box_sdk_gen.schemas.v2025_r0.client_error_v2025_r0 import ClientErrorV2025R0

from box_sdk_gen.schemas.v2025_r0.shield_lists_create_v2025_r0 import (
    ShieldListsCreateV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_lists_update_v2025_r0 import (
    ShieldListsUpdateV2025R0,
)

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class ShieldListsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_shield_lists_v2025_r0(
        self,
        *,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldListsV2025R0:
        """
        Retrieves all shield lists in the enterprise.
        :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
        :type box_version: BoxVersionHeaderV2025R0, optional
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/2.0/shield_lists']
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldListsV2025R0)

    def create_shield_list_v2025_r0(
        self,
        name: str,
        content: ShieldListContentRequestV2025R0,
        *,
        description: Optional[str] = None,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldListV2025R0:
        """
        Creates a shield list.
        :param name: The name of the shield list.
        :type name: str
        :param description: Optional description of Shield List., defaults to None
        :type description: Optional[str], optional
        :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
        :type box_version: BoxVersionHeaderV2025R0, optional
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'description': description,
            'content': content,
        }
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/2.0/shield_lists']
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldListV2025R0)

    def get_shield_list_by_id_v2025_r0(
        self,
        shield_list_id: str,
        *,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldListV2025R0:
        """
                Retrieves a single shield list by its ID.
                :param shield_list_id: The unique identifier that represents a shield list.
        The ID for any Shield List can be determined by the response from the endpoint
        fetching all shield lists for the enterprise.
        Example: "90fb0e17-c332-40ed-b4f9-fa8908fbbb24 "
                :type shield_list_id: str
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_lists/',
                        to_string(shield_list_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldListV2025R0)

    def delete_shield_list_by_id_v2025_r0(
        self,
        shield_list_id: str,
        *,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Delete a single shield list by its ID.
                :param shield_list_id: The unique identifier that represents a shield list.
        The ID for any Shield List can be determined by the response from the endpoint
        fetching all shield lists for the enterprise.
        Example: "90fb0e17-c332-40ed-b4f9-fa8908fbbb24 "
                :type shield_list_id: str
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_lists/',
                        to_string(shield_list_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def update_shield_list_by_id_v2025_r0(
        self,
        shield_list_id: str,
        name: str,
        content: ShieldListContentRequestV2025R0,
        *,
        description: Optional[str] = None,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldListV2025R0:
        """
                Updates a shield list.
                :param shield_list_id: The unique identifier that represents a shield list.
        The ID for any Shield List can be determined by the response from the endpoint
        fetching all shield lists for the enterprise.
        Example: "90fb0e17-c332-40ed-b4f9-fa8908fbbb24 "
                :type shield_list_id: str
                :param name: The name of the shield list.
                :type name: str
                :param description: Optional description of Shield List., defaults to None
                :type description: Optional[str], optional
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'description': description,
            'content': content,
        }
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_lists/',
                        to_string(shield_list_id),
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldListV2025R0)



from typing import Optional

from typing import List

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.legal_hold_policies import LegalHoldPolicies

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.legal_hold_policy import LegalHoldPolicy

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData

from box_sdk_gen.internal.utils import DateTime


class LegalHoldPoliciesManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_legal_hold_policies(
        self,
        *,
        policy_name: Optional[str] = None,
        fields: Optional[List[str]] = None,
        marker: Optional[str] = None,
        limit: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> LegalHoldPolicies:
        """
                Retrieves a list of legal hold policies that belong to

                an enterprise.

                :param policy_name: Limits results to policies for which the names start with
        this search term. This is a case-insensitive prefix., defaults to None
                :type policy_name: Optional[str], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'policy_name': to_string(policy_name),
                'fields': to_string(fields),
                'marker': to_string(marker),
                'limit': to_string(limit),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/legal_hold_policies',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, LegalHoldPolicies)

    def create_legal_hold_policy(
        self,
        policy_name: str,
        *,
        description: Optional[str] = None,
        filter_started_at: Optional[DateTime] = None,
        filter_ended_at: Optional[DateTime] = None,
        is_ongoing: Optional[bool] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> LegalHoldPolicy:
        """
                Create a new legal hold policy.
                :param policy_name: The name of the policy.
                :type policy_name: str
                :param description: A description for the policy., defaults to None
                :type description: Optional[str], optional
                :param filter_started_at: The filter start date.

        When this policy is applied using a `custodian` legal
        hold assignments, it will only apply to file versions
        created or uploaded inside of the
        date range. Other assignment types, such as folders and
        files, will ignore the date filter.

        Required if `is_ongoing` is set to `false`., defaults to None
                :type filter_started_at: Optional[DateTime], optional
                :param filter_ended_at: The filter end date.

        When this policy is applied using a `custodian` legal
        hold assignments, it will only apply to file versions
        created or uploaded inside of the
        date range. Other assignment types, such as folders and
        files, will ignore the date filter.

        Required if `is_ongoing` is set to `false`., defaults to None
                :type filter_ended_at: Optional[DateTime], optional
                :param is_ongoing: Whether new assignments under this policy should
        continue applying to files even after initialization.

        When this policy is applied using a legal hold assignment,
        it will continue applying the policy to any new file versions
        even after it has been applied.

        For example, if a legal hold assignment is placed on a user
        today, and that user uploads a file tomorrow, that file will
        get held. This will continue until the policy is retired.

        Required if no filter dates are set., defaults to None
                :type is_ongoing: Optional[bool], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'policy_name': policy_name,
            'description': description,
            'filter_started_at': filter_started_at,
            'filter_ended_at': filter_ended_at,
            'is_ongoing': is_ongoing,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/legal_hold_policies',
                    ]
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, LegalHoldPolicy)

    def get_legal_hold_policy_by_id(
        self,
        legal_hold_policy_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> LegalHoldPolicy:
        """
                Retrieve a legal hold policy.
                :param legal_hold_policy_id: The ID of the legal hold policy.
        Example: "324432"
                :type legal_hold_policy_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/legal_hold_policies/',
                        to_string(legal_hold_policy_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, LegalHoldPolicy)

    def update_legal_hold_policy_by_id(
        self,
        legal_hold_policy_id: str,
        *,
        policy_name: Optional[str] = None,
        description: Optional[str] = None,
        release_notes: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> LegalHoldPolicy:
        """
                Update legal hold policy.
                :param legal_hold_policy_id: The ID of the legal hold policy.
        Example: "324432"
                :type legal_hold_policy_id: str
                :param policy_name: The name of the policy., defaults to None
                :type policy_name: Optional[str], optional
                :param description: A description for the policy., defaults to None
                :type description: Optional[str], optional
                :param release_notes: Notes around why the policy was released., defaults to None
                :type release_notes: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'policy_name': policy_name,
            'description': description,
            'release_notes': release_notes,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/legal_hold_policies/',
                        to_string(legal_hold_policy_id),
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, LegalHoldPolicy)

    def delete_legal_hold_policy_by_id(
        self,
        legal_hold_policy_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Delete an existing legal hold policy.

                This is an asynchronous process. The policy will not be


                fully deleted yet when the response returns.

                :param legal_hold_policy_id: The ID of the legal hold policy.
        Example: "324432"
                :type legal_hold_policy_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/legal_hold_policies/',
                        to_string(legal_hold_policy_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None



from enum import Enum

from typing import Optional

from typing import Dict

from typing import List

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.metadata_templates import MetadataTemplates

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.metadata_template import MetadataTemplate

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class GetMetadataTemplateScope(str, Enum):
    GLOBAL = 'global'
    ENTERPRISE = 'enterprise'


class UpdateMetadataTemplateScope(str, Enum):
    GLOBAL = 'global'
    ENTERPRISE = 'enterprise'


class UpdateMetadataTemplateRequestBodyOpField(str, Enum):
    EDITTEMPLATE = 'editTemplate'
    ADDFIELD = 'addField'
    REORDERFIELDS = 'reorderFields'
    ADDENUMOPTION = 'addEnumOption'
    REORDERENUMOPTIONS = 'reorderEnumOptions'
    REORDERMULTISELECTOPTIONS = 'reorderMultiSelectOptions'
    ADDMULTISELECTOPTION = 'addMultiSelectOption'
    EDITFIELD = 'editField'
    REMOVEFIELD = 'removeField'
    EDITENUMOPTION = 'editEnumOption'
    REMOVEENUMOPTION = 'removeEnumOption'
    EDITMULTISELECTOPTION = 'editMultiSelectOption'
    REMOVEMULTISELECTOPTION = 'removeMultiSelectOption'


class UpdateMetadataTemplateRequestBody(BaseObject):
    _fields_to_json_mapping: Dict[str, str] = {
        'field_key': 'fieldKey',
        'field_keys': 'fieldKeys',
        'enum_option_key': 'enumOptionKey',
        'enum_option_keys': 'enumOptionKeys',
        'multi_select_option_key': 'multiSelectOptionKey',
        'multi_select_option_keys': 'multiSelectOptionKeys',
        **BaseObject._fields_to_json_mapping,
    }
    _json_to_fields_mapping: Dict[str, str] = {
        'fieldKey': 'field_key',
        'fieldKeys': 'field_keys',
        'enumOptionKey': 'enum_option_key',
        'enumOptionKeys': 'enum_option_keys',
        'multiSelectOptionKey': 'multi_select_option_key',
        'multiSelectOptionKeys': 'multi_select_option_keys',
        **BaseObject._json_to_fields_mapping,
    }

    def __init__(
        self,
        op: UpdateMetadataTemplateRequestBodyOpField,
        *,
        data: Optional[Dict] = None,
        field_key: Optional[str] = None,
        field_keys: Optional[List[str]] = None,
        enum_option_key: Optional[str] = None,
        enum_option_keys: Optional[List[str]] = None,
        multi_select_option_key: Optional[str] = None,
        multi_select_option_keys: Optional[List[str]] = None,
        **kwargs
    ):
        """
                :param op: The type of change to perform on the template. Some
        of these are hazardous as they will change existing templates.
                :type op: UpdateMetadataTemplateRequestBodyOpField
                :param data: The data for the operation. This will vary depending on the
        operation being performed., defaults to None
                :type data: Optional[Dict], optional
                :param field_key: For operations that affect a single field this defines the key of
        the field that is affected., defaults to None
                :type field_key: Optional[str], optional
                :param field_keys: For operations that affect multiple fields this defines the keys
        of the fields that are affected., defaults to None
                :type field_keys: Optional[List[str]], optional
                :param enum_option_key: For operations that affect a single `enum` option this defines
        the key of the option that is affected., defaults to None
                :type enum_option_key: Optional[str], optional
                :param enum_option_keys: For operations that affect multiple `enum` options this defines
        the keys of the options that are affected., defaults to None
                :type enum_option_keys: Optional[List[str]], optional
                :param multi_select_option_key: For operations that affect a single multi select option this
        defines the key of the option that is affected., defaults to None
                :type multi_select_option_key: Optional[str], optional
                :param multi_select_option_keys: For operations that affect multiple multi select options this
        defines the keys of the options that are affected., defaults to None
                :type multi_select_option_keys: Optional[List[str]], optional
        """
        super().__init__(**kwargs)
        self.op = op
        self.data = data
        self.field_key = field_key
        self.field_keys = field_keys
        self.enum_option_key = enum_option_key
        self.enum_option_keys = enum_option_keys
        self.multi_select_option_key = multi_select_option_key
        self.multi_select_option_keys = multi_select_option_keys


class DeleteMetadataTemplateScope(str, Enum):
    GLOBAL = 'global'
    ENTERPRISE = 'enterprise'


class CreateMetadataTemplateFieldsTypeField(str, Enum):
    STRING = 'string'
    FLOAT = 'float'
    DATE = 'date'
    ENUM = 'enum'
    MULTISELECT = 'multiSelect'
    TAXONOMY = 'taxonomy'


class CreateMetadataTemplateFieldsOptionsField(BaseObject):
    def __init__(self, key: str, **kwargs):
        """
                :param key: The text value of the option. This represents both the display name of the
        option and the internal key used when updating templates.
                :type key: str
        """
        super().__init__(**kwargs)
        self.key = key


class CreateMetadataTemplateFieldsOptionsRulesField(BaseObject):
    _fields_to_json_mapping: Dict[str, str] = {
        'multi_select': 'multiSelect',
        'selectable_levels': 'selectableLevels',
        **BaseObject._fields_to_json_mapping,
    }
    _json_to_fields_mapping: Dict[str, str] = {
        'multiSelect': 'multi_select',
        'selectableLevels': 'selectable_levels',
        **BaseObject._json_to_fields_mapping,
    }

    def __init__(
        self,
        *,
        multi_select: Optional[bool] = None,
        selectable_levels: Optional[List[int]] = None,
        **kwargs
    ):
        """
                :param multi_select: Whether to allow users to select multiple values., defaults to None
                :type multi_select: Optional[bool], optional
                :param selectable_levels: An array of integers defining which levels of the taxonomy are
        selectable by users., defaults to None
                :type selectable_levels: Optional[List[int]], optional
        """
        super().__init__(**kwargs)
        self.multi_select = multi_select
        self.selectable_levels = selectable_levels


class CreateMetadataTemplateFields(BaseObject):
    _fields_to_json_mapping: Dict[str, str] = {
        'display_name': 'displayName',
        'taxonomy_key': 'taxonomyKey',
        'options_rules': 'optionsRules',
        **BaseObject._fields_to_json_mapping,
    }
    _json_to_fields_mapping: Dict[str, str] = {
        'displayName': 'display_name',
        'taxonomyKey': 'taxonomy_key',
        'optionsRules': 'options_rules',
        **BaseObject._json_to_fields_mapping,
    }
    _discriminator = 'type', {
        'string',
        'float',
        'date',
        'enum',
        'multiSelect',
        'taxonomy',
    }

    def __init__(
        self,
        type: CreateMetadataTemplateFieldsTypeField,
        key: str,
        display_name: str,
        *,
        description: Optional[str] = None,
        hidden: Optional[bool] = None,
        options: Optional[List[CreateMetadataTemplateFieldsOptionsField]] = None,
        taxonomy_key: Optional[str] = None,
        namespace: Optional[str] = None,
        options_rules: Optional[CreateMetadataTemplateFieldsOptionsRulesField] = None,
        **kwargs
    ):
        """
                :param type: The type of field. The basic fields are a `string` field for text, a
        `float` field for numbers, and a `date` field to present the user with a
        date-time picker.

        Additionally, metadata templates support an `enum` field for a basic list
        of items, and ` multiSelect` field for a similar list of items where the
        user can select more than one value.

        Metadata taxonomies are also supported as a `taxonomy` field type
        with a specific set of additional properties, which describe its structure.
                :type type: CreateMetadataTemplateFieldsTypeField
                :param key: A unique identifier for the field. The identifier must
        be unique within the template to which it belongs.
                :type key: str
                :param display_name: The display name of the field as it is shown to the user in the web and
        mobile apps.
                :type display_name: str
                :param description: A description of the field. This is not shown to the user., defaults to None
                :type description: Optional[str], optional
                :param hidden: Whether this field is hidden in the UI for the user and can only be set
        through the API instead., defaults to None
                :type hidden: Optional[bool], optional
                :param options: A list of options for this field. This is used in combination with the
        `enum` and `multiSelect` field types., defaults to None
                :type options: Optional[List[CreateMetadataTemplateFieldsOptionsField]], optional
                :param taxonomy_key: The unique key of the metadata taxonomy to use for this taxonomy field.
        This property is required when the field `type` is set to `taxonomy`., defaults to None
                :type taxonomy_key: Optional[str], optional
                :param namespace: The namespace of the metadata taxonomy to use for this taxonomy field.
        This property is required when the field `type` is set to `taxonomy`., defaults to None
                :type namespace: Optional[str], optional
                :param options_rules: An object defining additional rules for the options of the taxonomy field.
        This property is required when the field `type` is set to `taxonomy`., defaults to None
                :type options_rules: Optional[CreateMetadataTemplateFieldsOptionsRulesField], optional
        """
        super().__init__(**kwargs)
        self.type = type
        self.key = key
        self.display_name = display_name
        self.description = description
        self.hidden = hidden
        self.options = options
        self.taxonomy_key = taxonomy_key
        self.namespace = namespace
        self.options_rules = options_rules


class MetadataTemplatesManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_metadata_templates_by_instance_id(
        self,
        metadata_instance_id: str,
        *,
        marker: Optional[str] = None,
        limit: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplates:
        """
                Finds a metadata template by searching for the ID of an instance of the

                template.

                :param metadata_instance_id: The ID of an instance of the metadata template to find.
                :type metadata_instance_id: str
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'metadata_instance_id': to_string(metadata_instance_id),
                'marker': to_string(marker),
                'limit': to_string(limit),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/2.0/metadata_templates']
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplates)

    def get_metadata_template(
        self,
        scope: GetMetadataTemplateScope,
        template_key: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplate:
        """
                Retrieves a metadata template by its `scope` and `templateKey` values.

                To find the `scope` and `templateKey` for a template, list all templates for


                an enterprise or globally, or list all templates applied to a file or folder.

                :param scope: The scope of the metadata template.
        Example: "global"
                :type scope: GetMetadataTemplateScope
                :param template_key: The name of the metadata template.
        Example: "properties"
                :type template_key: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/',
                        to_string(scope),
                        '/',
                        to_string(template_key),
                        '/schema',
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplate)

    def update_metadata_template(
        self,
        scope: UpdateMetadataTemplateScope,
        template_key: str,
        request_body: List[UpdateMetadataTemplateRequestBody],
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplate:
        """
                Updates a metadata template.

                The metadata template can only be updated if the template


                already exists.


                The update is applied atomically. If any errors occur during the


                application of the operations, the metadata template will not be changed.

                :param scope: The scope of the metadata template.
        Example: "global"
                :type scope: UpdateMetadataTemplateScope
                :param template_key: The name of the metadata template.
        Example: "properties"
                :type template_key: str
                :param request_body: Request body of updateMetadataTemplate method
                :type request_body: List[UpdateMetadataTemplateRequestBody]
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/',
                        to_string(scope),
                        '/',
                        to_string(template_key),
                        '/schema',
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json-patch+json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplate)

    def delete_metadata_template(
        self,
        scope: DeleteMetadataTemplateScope,
        template_key: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Delete a metadata template and its instances.

                This deletion is permanent and can not be reversed.

                :param scope: The scope of the metadata template.
        Example: "global"
                :type scope: DeleteMetadataTemplateScope
                :param template_key: The name of the metadata template.
        Example: "properties"
                :type template_key: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/',
                        to_string(scope),
                        '/',
                        to_string(template_key),
                        '/schema',
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def get_metadata_template_by_id(
        self,
        template_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplate:
        """
                Retrieves a metadata template by its ID.
                :param template_id: The ID of the template.
        Example: "f7a9891f"
                :type template_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/',
                        to_string(template_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplate)

    def get_global_metadata_templates(
        self,
        *,
        marker: Optional[str] = None,
        limit: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplates:
        """
                Used to retrieve all generic, global metadata templates available to all

                enterprises using Box.

                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {'marker': to_string(marker), 'limit': to_string(limit)}
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/global',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplates)

    def get_enterprise_metadata_templates(
        self,
        *,
        marker: Optional[str] = None,
        limit: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplates:
        """
                Used to retrieve all metadata templates created to be used specifically within

                the user's enterprise.

                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {'marker': to_string(marker), 'limit': to_string(limit)}
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/enterprise',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplates)

    def create_metadata_template(
        self,
        scope: str,
        display_name: str,
        *,
        template_key: Optional[str] = None,
        hidden: Optional[bool] = None,
        fields: Optional[List[CreateMetadataTemplateFields]] = None,
        copy_instance_on_item_copy: Optional[bool] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> MetadataTemplate:
        """
                Creates a new metadata template that can be applied to

                files and folders.

                :param scope: The scope of the metadata template to create. Applications can
        only create templates for use within the authenticated user's
        enterprise.

        This value needs to be set to `enterprise`, as `global` scopes can
        not be created by applications.
                :type scope: str
                :param display_name: The display name of the template.
                :type display_name: str
                :param template_key: A unique identifier for the template. This identifier needs to be
        unique across the enterprise for which the metadata template is
        being created.

        When not provided, the API will create a unique `templateKey`
        based on the value of the `displayName`., defaults to None
                :type template_key: Optional[str], optional
                :param hidden: Defines if this template is visible in the Box web app UI, or if
        it is purely intended for usage through the API., defaults to None
                :type hidden: Optional[bool], optional
                :param fields: An ordered list of template fields which are part of the template.
        Each field can be a regular text field, date field, number field,
        as well as a single or multi-select list., defaults to None
                :type fields: Optional[List[CreateMetadataTemplateFields]], optional
                :param copy_instance_on_item_copy: Whether or not to copy any metadata attached to a file or folder
        when it is copied. By default, metadata is not copied along with a
        file or folder when it is copied., defaults to None
                :type copy_instance_on_item_copy: Optional[bool], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'scope': scope,
            'templateKey': template_key,
            'displayName': display_name,
            'hidden': hidden,
            'fields': fields,
            'copyInstanceOnItemCopy': copy_instance_on_item_copy,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/metadata_templates/schema',
                    ]
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, MetadataTemplate)



from typing import Optional

from typing import List

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.v2025_r0.shield_rule_item_v2025_r0 import ShieldRuleItemV2025R0

from box_sdk_gen.box.errors import BoxSDKError


class EnterpriseConfigurationShieldV2025R0(BaseObject):
    def __init__(
        self, *, shield_rules: Optional[List[ShieldRuleItemV2025R0]] = None, **kwargs
    ):
        """
        :param shield_rules: The shield rules configuration for the enterprise., defaults to None
        :type shield_rules: Optional[List[ShieldRuleItemV2025R0]], optional
        """
        super().__init__(**kwargs)
        self.shield_rules = shield_rules



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class WorkflowMiniTypeField(str, Enum):
    WORKFLOW = 'workflow'


class WorkflowMini(BaseObject):
    _discriminator = 'type', {'workflow'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[WorkflowMiniTypeField] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        is_enabled: Optional[bool] = None,
        **kwargs
    ):
        """
        :param id: The unique identifier for the workflow., defaults to None
        :type id: Optional[str], optional
        :param type: The value will always be `workflow`., defaults to None
        :type type: Optional[WorkflowMiniTypeField], optional
        :param name: The name of the workflow., defaults to None
        :type name: Optional[str], optional
        :param description: The description for a workflow., defaults to None
        :type description: Optional[str], optional
        :param is_enabled: Specifies if this workflow is enabled., defaults to None
        :type is_enabled: Optional[bool], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.name = name
        self.description = description
        self.is_enabled = is_enabled



from enum import Enum

from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.schemas.post_o_auth_2_token import PostOAuth2TokenGrantTypeField

from box_sdk_gen.schemas.post_o_auth_2_token import PostOAuth2TokenSubjectTokenTypeField

from box_sdk_gen.schemas.post_o_auth_2_token import PostOAuth2TokenBoxSubjectTypeField

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.access_token import AccessToken

from box_sdk_gen.schemas.o_auth_2_error import OAuth2Error

from box_sdk_gen.schemas.post_o_auth_2_token import PostOAuth2Token

from box_sdk_gen.schemas.post_o_auth_2_token_refresh_access_token import (
    PostOAuth2TokenRefreshAccessToken,
)

from box_sdk_gen.schemas.post_o_auth_2_revoke import PostOAuth2Revoke

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class AuthorizeUserResponseType(str, Enum):
    CODE = 'code'


class RequestAccessTokenGrantType(str, Enum):
    AUTHORIZATION_CODE = 'authorization_code'
    REFRESH_TOKEN = 'refresh_token'
    CLIENT_CREDENTIALS = 'client_credentials'
    URN_IETF_PARAMS_OAUTH_GRANT_TYPE_JWT_BEARER = (
        'urn:ietf:params:oauth:grant-type:jwt-bearer'
    )
    URN_IETF_PARAMS_OAUTH_GRANT_TYPE_TOKEN_EXCHANGE = (
        'urn:ietf:params:oauth:grant-type:token-exchange'
    )


class RequestAccessTokenSubjectTokenType(str, Enum):
    URN_IETF_PARAMS_OAUTH_TOKEN_TYPE_ACCESS_TOKEN = (
        'urn:ietf:params:oauth:token-type:access_token'
    )


class RequestAccessTokenActorTokenType(str, Enum):
    URN_IETF_PARAMS_OAUTH_TOKEN_TYPE_ID_TOKEN = (
        'urn:ietf:params:oauth:token-type:id_token'
    )


class RequestAccessTokenBoxSubjectType(str, Enum):
    ENTERPRISE = 'enterprise'
    USER = 'user'


class RefreshAccessTokenGrantType(str, Enum):
    REFRESH_TOKEN = 'refresh_token'


class AuthorizationManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def authorize_user(
        self,
        response_type: AuthorizeUserResponseType,
        client_id: str,
        *,
        redirect_uri: Optional[str] = None,
        state: Optional[str] = None,
        scope: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Authorize a user by sending them through the [Box](https://box.com)

                website and request their permission to act on their behalf.


                This is the first step when authenticating a user using


                OAuth 2.0. To request a user's authorization to use the Box APIs


                on their behalf you will need to send a user to the URL with this


                format.

                :param response_type: The type of response we'd like to receive.
                :type response_type: AuthorizeUserResponseType
                :param client_id: The Client ID of the application that is requesting to authenticate
        the user. To get the Client ID for your application, log in to your
        Box developer console and click the **Edit Application** link for
        the application you're working with. In the OAuth 2.0 Parameters section
        of the configuration page, find the item labelled `client_id`. The
        text of that item is your application's Client ID.
                :type client_id: str
                :param redirect_uri: The URI to which Box redirects the browser after the user has granted
        or denied the application permission. This URI match one of the redirect
        URIs in the configuration of your application. It must be a
        valid HTTPS URI and it needs to be able to handle the redirection to
        complete the next step in the OAuth 2.0 flow.
        Although this parameter is optional, it must be a part of the
        authorization URL if you configured multiple redirect URIs
        for the application in the developer console. A missing parameter causes
        a `redirect_uri_missing` error after the user grants application access., defaults to None
                :type redirect_uri: Optional[str], optional
                :param state: A custom string of your choice. Box will pass the same string to
        the redirect URL when authentication is complete. This parameter
        can be used to identify a user on redirect, as well as protect
        against hijacked sessions and other exploits., defaults to None
                :type state: Optional[str], optional
                :param scope: A space-separated list of application scopes you'd like to
        authenticate the user for. This defaults to all the scopes configured
        for the application in its configuration page., defaults to None
                :type scope: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'response_type': to_string(response_type),
                'client_id': to_string(client_id),
                'redirect_uri': to_string(redirect_uri),
                'state': to_string(state),
                'scope': to_string(scope),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.oauth_2_url, '/authorize']),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def request_access_token(
        self,
        grant_type: RequestAccessTokenGrantType,
        *,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        code: Optional[str] = None,
        refresh_token: Optional[str] = None,
        assertion: Optional[str] = None,
        subject_token: Optional[str] = None,
        subject_token_type: Optional[RequestAccessTokenSubjectTokenType] = None,
        actor_token: Optional[str] = None,
        actor_token_type: Optional[RequestAccessTokenActorTokenType] = None,
        scope: Optional[str] = None,
        resource: Optional[str] = None,
        box_subject_type: Optional[RequestAccessTokenBoxSubjectType] = None,
        box_subject_id: Optional[str] = None,
        box_shared_link: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> AccessToken:
        """
                Request an Access Token using either a client-side obtained OAuth 2.0

                authorization code or a server-side JWT assertion.


                An Access Token is a string that enables Box to verify that a


                request belongs to an authorized session. In the normal order of


                operations you will begin by requesting authentication from the


                [authorize](https://developer.box.com/reference/get-authorize) endpoint and Box will send you an


                authorization code.


                You will then send this code to this endpoint to exchange it for


                an Access Token. The returned Access Token can then be used to to make


                Box API calls.

                :param grant_type: The type of request being made, either using a client-side obtained
        authorization code, a refresh token, a JWT assertion, client credentials
        grant or another access token for the purpose of downscoping a token.
                :type grant_type: RequestAccessTokenGrantType
                :param client_id: The Client ID of the application requesting an access token.

        Used in combination with `authorization_code`, `client_credentials`, or
        `urn:ietf:params:oauth:grant-type:jwt-bearer` as the `grant_type`., defaults to None
                :type client_id: Optional[str], optional
                :param client_secret: The client secret of the application requesting an access token.

        Used in combination with `authorization_code`, `client_credentials`, or
        `urn:ietf:params:oauth:grant-type:jwt-bearer` as the `grant_type`., defaults to None
                :type client_secret: Optional[str], optional
                :param code: The client-side authorization code passed to your application by
        Box in the browser redirect after the user has successfully
        granted your application permission to make API calls on their
        behalf.

        Used in combination with `authorization_code` as the `grant_type`., defaults to None
                :type code: Optional[str], optional
                :param refresh_token: A refresh token used to get a new access token with.

        Used in combination with `refresh_token` as the `grant_type`., defaults to None
                :type refresh_token: Optional[str], optional
                :param assertion: A JWT assertion for which to request a new access token.

        Used in combination with `urn:ietf:params:oauth:grant-type:jwt-bearer`
        as the `grant_type`., defaults to None
                :type assertion: Optional[str], optional
                :param subject_token: The token to exchange for a downscoped token. This can be a regular
        access token, a JWT assertion, or an app token.

        Used in combination with `urn:ietf:params:oauth:grant-type:token-exchange`
        as the `grant_type`., defaults to None
                :type subject_token: Optional[str], optional
                :param subject_token_type: The type of `subject_token` passed in.

        Used in combination with `urn:ietf:params:oauth:grant-type:token-exchange`
        as the `grant_type`., defaults to None
                :type subject_token_type: Optional[RequestAccessTokenSubjectTokenType], optional
                :param actor_token: The token used to create an annotator token.
        This is a JWT assertion.

        Used in combination with `urn:ietf:params:oauth:grant-type:token-exchange`
        as the `grant_type`., defaults to None
                :type actor_token: Optional[str], optional
                :param actor_token_type: The type of `actor_token` passed in.

        Used in combination with `urn:ietf:params:oauth:grant-type:token-exchange`
        as the `grant_type`., defaults to None
                :type actor_token_type: Optional[RequestAccessTokenActorTokenType], optional
                :param scope: The space-delimited list of scopes that you want apply to the
        new access token.

        The `subject_token` will need to have all of these scopes or
        the call will error with **401 Unauthorized**.., defaults to None
                :type scope: Optional[str], optional
                :param resource: Full URL for the file that the token should be generated for., defaults to None
                :type resource: Optional[str], optional
                :param box_subject_type: Used in combination with `client_credentials` as the `grant_type`., defaults to None
                :type box_subject_type: Optional[RequestAccessTokenBoxSubjectType], optional
                :param box_subject_id: Used in combination with `client_credentials` as the `grant_type`.
        Value is determined by `box_subject_type`. If `user` use user ID and if
        `enterprise` use enterprise ID., defaults to None
                :type box_subject_id: Optional[str], optional
                :param box_shared_link: Full URL of the shared link on the file or folder
        that the token should be generated for., defaults to None
                :type box_shared_link: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'grant_type': grant_type,
            'client_id': client_id,
            'client_secret': client_secret,
            'code': code,
            'refresh_token': refresh_token,
            'assertion': assertion,
            'subject_token': subject_token,
            'subject_token_type': subject_token_type,
            'actor_token': actor_token,
            'actor_token_type': actor_token_type,
            'scope': scope,
            'resource': resource,
            'box_subject_type': box_subject_type,
            'box_subject_id': box_subject_id,
            'box_shared_link': box_shared_link,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/oauth2/token']),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/x-www-form-urlencoded',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, AccessToken)

    def refresh_access_token(
        self,
        client_id: str,
        client_secret: str,
        refresh_token: str,
        *,
        grant_type: RefreshAccessTokenGrantType = RefreshAccessTokenGrantType.REFRESH_TOKEN,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> AccessToken:
        """
        Refresh an Access Token using its client ID, secret, and refresh token.
        :param client_id: The client ID of the application requesting to refresh the token.
        :type client_id: str
        :param client_secret: The client secret of the application requesting to refresh the token.
        :type client_secret: str
        :param refresh_token: The refresh token to refresh.
        :type refresh_token: str
        :param grant_type: The type of request being made, in this case a refresh request., defaults to RefreshAccessTokenGrantType.REFRESH_TOKEN
        :type grant_type: RefreshAccessTokenGrantType, optional
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'grant_type': grant_type,
            'client_id': client_id,
            'client_secret': client_secret,
            'refresh_token': refresh_token,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/oauth2/token#refresh']
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/x-www-form-urlencoded',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, AccessToken)

    def revoke_access_token(
        self,
        *,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        token: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Revoke an active Access Token, effectively logging a user out

                that has been previously authenticated.

                :param client_id: The Client ID of the application requesting to revoke the
        access token., defaults to None
                :type client_id: Optional[str], optional
                :param client_secret: The client secret of the application requesting to revoke
        an access token., defaults to None
                :type client_secret: Optional[str], optional
                :param token: The access token to revoke., defaults to None
                :type token: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'client_id': client_id,
            'client_secret': client_secret,
            'token': token,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/oauth2/revoke']
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/x-www-form-urlencoded',
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None



from typing import Dict

from ..internal.logging import DataSanitizer
from .network_client import NetworkClient
from .box_network_client import BoxNetworkClient
from .proxy_config import ProxyConfig
from .base_urls import BaseUrls
from .retries import RetryStrategy, BoxRetryStrategy
from .timeout_config import TimeoutConfig


class NetworkSession:
    def __init__(
        self,
        *,
        network_client: NetworkClient = None,
        retry_strategy: RetryStrategy = None,
        additional_headers: Dict[str, str] = None,
        base_urls: BaseUrls = None,
        proxy_url: str = None,
        data_sanitizer: DataSanitizer = None,
        timeout_config: TimeoutConfig = None,
    ):
        if additional_headers is None:
            additional_headers = {}
        if base_urls is None:
            base_urls = BaseUrls()
        if retry_strategy is None:
            retry_strategy = BoxRetryStrategy()
        if network_client is None:
            network_client = BoxNetworkClient()
        if (
            proxy_url
            and hasattr(network_client, 'requests_session')
            and network_client.requests_session
        ):
            network_client.requests_session.proxies = {
                'http': proxy_url,
                'https': proxy_url,
            }
        if data_sanitizer is None:
            data_sanitizer = DataSanitizer()
        if timeout_config is None:
            timeout_config = TimeoutConfig(
                connection_timeout_ms=5000,
                read_timeout_ms=60000,
            )
        self.additional_headers = additional_headers
        self.base_urls = base_urls
        self.proxy_url = proxy_url
        self.network_client = network_client
        self.retry_strategy = retry_strategy
        self.data_sanitizer = data_sanitizer
        self.timeout_config = timeout_config

    def with_additional_headers(
        self, additional_headers: Dict[str, str] = None
    ) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including additional headers to be attached to every API call.
        :param additional_headers: Dict of headers, which are appended to each API request
        :return: a new instance of NetworkSession
        """
        return NetworkSession(
            network_client=self.network_client,
            additional_headers={**self.additional_headers, **additional_headers},
            base_urls=self.base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_custom_base_urls(self, base_urls: BaseUrls) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including additional base urls to be used for each API call.
        :param base_urls: Dict of base urls, which are appended to each API request
        :return: a new instance of NetworkSession
        """
        return NetworkSession(
            network_client=self.network_client,
            additional_headers=self.additional_headers,
            base_urls=base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_proxy(self, config: ProxyConfig) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including a proxy to be used for each API call.
        :param config: ProxyConfig object, which contains the proxy url, username, and password
        :return: a new instance of NetworkSession
        """
        if not config.url or not config.url.startswith("http"):
            raise ValueError("Invalid proxy URL provided")

        proxy_host = config.url.split("//")[1]
        proxy_auth = (
            f"{config.username}:{config.password}@"
            if config.username and config.password
            else ""
        )
        proxy_url = f"http://{proxy_auth}{proxy_host}"
        return NetworkSession(
            network_client=self.network_client,
            additional_headers=self.additional_headers,
            base_urls=self.base_urls,
            proxy_url=proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_network_client(self, network_client: NetworkClient) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including a new network client to be used for each API call.
        :param network_client: NetworkClient object, which contains the fetch method
        :return: a new instance of NetworkSession
        """
        return NetworkSession(
            network_client=network_client,
            additional_headers=self.additional_headers,
            base_urls=self.base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_retry_strategy(self, retry_strategy: RetryStrategy) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including a new retry options to be used for each API call.
        :param retry_strategy: RetryStrategy object, which contains the retry logic
        :return: a new instance of NetworkSession
        """
        return NetworkSession(
            network_client=self.network_client,
            additional_headers=self.additional_headers,
            base_urls=self.base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_data_sanitizer(self, data_sanitizer: DataSanitizer) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also applying data sanitizer to sanitize sensitive data for logging.
        :param data_sanitizer:
        :return:
        """
        return NetworkSession(
            network_client=self.network_client,
            additional_headers=self.additional_headers,
            base_urls=self.base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=data_sanitizer,
            timeout_config=self.timeout_config,
        )

    def with_timeout_config(self, timeout_config: TimeoutConfig) -> 'NetworkSession':
        """
        Generate a fresh network session by duplicating the existing configuration and network parameters,
        while also including timeout config to be used for every API call.
        :param timeout_config: TimeoutConfig object, which contains the timeout config
        :return: a new instance of NetworkSession
        """
        return NetworkSession(
            network_client=self.network_client,
            additional_headers=self.additional_headers,
            base_urls=self.base_urls,
            proxy_url=self.proxy_url,
            retry_strategy=self.retry_strategy,
            data_sanitizer=self.data_sanitizer,
            timeout_config=timeout_config,
        )



from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from box_sdk_gen.schemas.v2025_r0.shield_list_content_country_v2025_r0 import (
    ShieldListContentCountryV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_domain_v2025_r0 import (
    ShieldListContentDomainV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_email_v2025_r0 import (
    ShieldListContentEmailV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_ip_v2025_r0 import (
    ShieldListContentIpV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_request_v2025_r0 import (
    ShieldListContentRequestV2025R0,
)

from box_sdk_gen.box.errors import BoxSDKError


class ShieldListsCreateV2025R0(BaseObject):
    def __init__(
        self,
        name: str,
        content: ShieldListContentRequestV2025R0,
        *,
        description: Optional[str] = None,
        **kwargs
    ):
        """
        :param name: The name of the shield list.
        :type name: str
        :param description: Optional description of Shield List., defaults to None
        :type description: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.name = name
        self.content = content
        self.description = description



from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from box_sdk_gen.box.errors import BoxSDKError


class AiAgentInfoModelsField(BaseObject):
    def __init__(
        self,
        *,
        name: Optional[str] = None,
        provider: Optional[str] = None,
        supported_purpose: Optional[str] = None,
        **kwargs
    ):
        """
        :param name: The name of the model used for the request., defaults to None
        :type name: Optional[str], optional
        :param provider: The provider that owns the model used for the request., defaults to None
        :type provider: Optional[str], optional
        :param supported_purpose: The supported purpose utilized by the model used for the request., defaults to None
        :type supported_purpose: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.name = name
        self.provider = provider
        self.supported_purpose = supported_purpose


class AiAgentInfo(BaseObject):
    def __init__(
        self,
        *,
        models: Optional[List[AiAgentInfoModelsField]] = None,
        processor: Optional[str] = None,
        **kwargs
    ):
        """
        :param models: The models used for the request., defaults to None
        :type models: Optional[List[AiAgentInfoModelsField]], optional
        :param processor: The processor used for the request., defaults to None
        :type processor: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.models = models
        self.processor = processor



from typing import Optional

from typing import Dict

from box_sdk_gen.serialization.json import SerializedData

from box_sdk_gen.internal.utils import ByteStream


class FetchResponse:
    def __init__(
        self,
        status: int,
        headers: Dict[str, str],
        *,
        url: Optional[str] = None,
        data: Optional[SerializedData] = None,
        content: Optional[ByteStream] = None
    ):
        """
        :param status: HTTP status code of the response
        :type status: int
        :param headers: HTTP headers of the response
        :type headers: Dict[str, str]
        :param url: URL of the response, defaults to None
        :type url: Optional[str], optional
        :param data: Response body of the response, defaults to None
        :type data: Optional[SerializedData], optional
        :param content: Streamed content of the response, defaults to None
        :type content: Optional[ByteStream], optional
        """
        self.status = status
        self.headers = headers
        self.url = url
        self.data = data
        self.content = content
